/*
 * Copyright 2016 ELVEES NeoTek JSC
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#ifndef _LINUX_ELCORE40_H
#define _LINUX_ELCORE40_H

#include <stddef.h>

#define MAX_CORES 1
#define MAX_VDMAS 2
#define MAX_EVENTS 1

#define MAX_KERNELS 1
#define MAX_INPUTS 15
#define MAX_OUTPUTS 15

enum elcore_memory_type {
	ELCORE_MEMORY_CODE,
	ELCORE_MEMORY_VRAM,
	ELCORE_MEMORY_SYSTEM
};

struct elcore_buffer {
	int fd;
	enum elcore_memory_type type;
	size_t size;
};

enum elcore_resource_type {
	ELCORE_CORE,
	ELCORE_VDMA,
	ELCORE_EVENT
};

struct elcore_resource {
	enum elcore_resource_type type;
	int descriptor;
};

enum elcore_job_status {
	ELCORE_JOB_IDLE,
	ELCORE_JOB_ENQUEUED,
	ELCORE_JOB_RUNNING,
};

enum elcore_job_rc {
	ELCORE_JOB_ERROR = -2,
	ELCORE_JOB_CANCELLED = -1,
	ELCORE_JOB_SUCCESS = 0
};

#define ELCORE_INIT_STACK (1 << 0)

struct elcore_job {
	int fd;

	int inum; /* actual number of input arguments */
	int onum; /* actual number of output arguments */

	int kernel[MAX_KERNELS];
	int input[MAX_INPUTS];
	int output[MAX_OUTPUTS];

	unsigned int flags;

	struct elcore_resource cores[MAX_CORES];
	struct elcore_resource vdmas[MAX_VDMAS];
	struct elcore_resource events[MAX_EVENTS];

	enum elcore_job_status status;
	enum elcore_job_rc rc;
};

/**
 * struct elcore_dmachain - DMA chain information
 * @external: File descriptors of external (system) buffers.
 * @internal: File descriptors of internal (VRAM) buffers.
 * @chain: File descriptor of buffer with chained DMA descriptors.
 */
struct elcore_dmachain {
	int external;
	int internal[2];
	int chain;
};

#define ELCIOC_MAGIC 'e'

#define ELCIOC_BUF_ALLOC \
	_IOWR(ELCIOC_MAGIC, 1, struct elcore_buffer *)

#define ELCIOC_RESOURCE_REQUEST \
	_IOWR(ELCIOC_MAGIC, 2, struct elcore_resource *)

#define ELCIOC_RESOURCE_RELEASE \
	_IOW(ELCIOC_MAGIC, 3, struct elcore_resource *)

#define ELCIOC_JOB_CREATE \
	_IOWR(ELCIOC_MAGIC, 4, struct elcore_job *)

#define ELCIOC_JOB_ENQUEUE \
	_IOW(ELCIOC_MAGIC, 5, struct elcore_job *)

#define ELCIOC_JOB_STATUS \
	_IOWR(ELCIOC_MAGIC, 6, struct elcore_job *)

#define ELCIOC_JOB_CANCEL \
	_IOW(ELCIOC_MAGIC, 7, struct elcore_job *)

#define ELCIOC_DMACHAIN_SETUP \
	_IOR(ELCIOC_MAGIC, 8, struct elcore_dmachain *)

#endif
