/*
 * Copyright 2016 ELVEES NeoTek JSC
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#ifndef _VDMA_H_
#define _VDMA_H_

#include <linux/types.h>
#include <linux/bitops.h>

#define VDMA_CHAN_A0E                 0x00
#define VDMA_CHAN_AECUR               0x04
#define VDMA_CHAN_A0I                 0x08
#define VDMA_CHAN_AICUR               0x0C
#define VDMA_CHAN_BEIDX               0x10
#define VDMA_CHAN_CEIDX               0x14
#define VDMA_CHAN_HEIDX               0x18
#define VDMA_CHAN_VEIDX               0x1c
#define VDMA_CHAN_BIIDX               0x20
#define VDMA_CHAN_CIIDX               0x24
#define VDMA_CHAN_HIIDX               0x28
#define VDMA_CHAN_VIIDX               0x2c
#define VDMA_CHAN_ACNT                0x30
#define VDMA_CHAN_BCCNT               0x34
#define VDMA_CHAN_HVECNT              0x38
#define VDMA_CHAN_HVICNT              0x3C
#define VDMA_CHAN_RUN                 0x40
#define VDMA_CHAN_DONE                0x44
#define VDMA_CHAN_IMRDY               0x48
#define VDMA_CHAN_INIT                0x4c
#define VDMA_CHAN_IRQ                 0x50
#define VDMA_CHAN_IRQM                0x54
#define VDMA_CHAN_CFG                 0x58
#define VDMA_CHAN_CSR                 0x5C
#define VDMA_CHAN_AINIT               0x68

#define VDMA_CTRL_BASE 0x800
#define VDMA_ENABLE    (VDMA_CTRL_BASE + 0x00)
#define VDMA_BUSY      (VDMA_CTRL_BASE + 0x04)
#define VDMA_ACTV      (VDMA_CTRL_BASE + 0x08)
#define VDMA_DONE      (VDMA_CTRL_BASE + 0x0c)
#define VDMA_IMEM_RDY  (VDMA_CTRL_BASE + 0x10)
#define VDMA_MODE      (VDMA_CTRL_BASE + 0x14)
#define VDMA_INIT      (VDMA_CTRL_BASE + 0x18)
#define VDMA_IRQ       (VDMA_CTRL_BASE + 0x1c)
#define VDMA_IRQM      (VDMA_CTRL_BASE + 0x20)

enum {
	VDMA_BURST_FIXED,
	VDMA_BURST_INCR
};

enum {
	VDMA_SIZE_1,
	VDMA_SIZE_2,
	VDMA_SIZE_4,
	VDMA_SIZE_8,
	VDMA_SIZE_16
};

enum vdma_dir {
	VDMA_DIR_TO_VRAM,
	VDMA_DIR_FROM_VRAM
};

#define VDMA_CFG_DIR                  BIT(0)
#define VDMA_CFG_DIM                  BIT(1)
#define VDMA_CFG_CYCLE                BIT(2)
#define VDMA_CFG_PRT                  GENMASK(6, 3)
#define VDMA_CFG_BRST_AE              BIT(7)
#define VDMA_CFG_BRST_AI              BIT(8)
#define VDMA_CFG_SIZE                 GENMASK(11, 9)
#define VDMA_CFG_BC_DONE              BIT(12)
#define VDMA_CFG_DONE_MSK             BIT(13)

#define VDMA_CSR_ENABLE               BIT(0)
#define VDMA_CSR_DONE                 BIT(1)
#define VDMA_CSR_IMEM_RDY             BIT(2)
#define VDMA_CSR_INITEN               BIT(3)
#define VDMA_CSR_INITDIR              BIT(4)
#define VDMA_CSR_IRQM                 BIT(5)
#define VDMA_CSR_BUSY                 BIT(6)
#define VDMA_CSR_ACTV                 BIT(7)

#define VDMA_ACNT_ACNT                GENMASK(15, 0)
#define VDMA_ACNT_ARLD                GENMASK(31, 16)

#define VDMA_BCCNT_BCNT               GENMASK(7, 0)
#define VDMA_BCCNT_CCNT               GENMASK(15, 8)
#define VDMA_BCCNT_BRLD               GENMASK(23, 16)
#define VDMA_BCCNT_CRLD               GENMASK(31, 24)

#define VDMA_HVXCNT_HCNT              GENMASK(7, 0)
#define VDMA_HVXCNT_VCNT              GENMASK(15, 8)
#define VDMA_HVXCNT_HRLD              GENMASK(23, 16)
#define VDMA_HVXCNT_VRLD              GENMASK(31, 24)

struct vdma_descriptor {
	u32 a0e, aecur;
	u32 a0i, aicur;
	u32 beidx, ceidx, heidx, veidx;
	u32 biidx, ciidx, hiidx, viidx;
	u16 acnt, arld;
	u8 bcnt, brld, ccnt, crld;
	u8 hecnt, herld, vecnt, verld;
	u8 hicnt, hirld, vicnt, virld;
	u32 cfg;
	u32 a_init;
	u32 reserved[2];
	u32 csr;
} __aligned(16);

#ifndef FIELD_PREP /* These can be defined elsewhere */

#define _bitfield_shift(x) (__builtin_ffsll(x) - 1)

#define FIELD_PREP(mask, val) \
	(((typeof(mask))(val) << _bitfield_shift(mask)) & (mask))

#define FIELD_GET(mask, reg) \
	((typeof(mask))(((reg) & (mask)) >> _bitfield_shift(mask)))

#endif

int vdma_request_channel(void);
void vdma_release_channel(int channel);
void vdma_write(int channel, u32 reg, u32 data);
u32 vdma_read(int channel, u32 reg);

void vdma_enable(unsigned int channels, int cids[]);
void vdma_wait_all(unsigned int channels, int cids[]);
void vdma_wait_all_alt(unsigned int channels, int cids[]);

void vdma_dump_channel(char const *const level, int channel);
void vdma_print_descriptor(char const *const level,
			   struct vdma_descriptor const *const vd);

#endif
